import * as z from 'zod';
import { Workflow } from './workflow';
import { Action } from './clients/dispatcher/action-listener';
import { LogLevel } from './clients/event/event-client';
import { Logger } from './util/logger';
import { HatchetClient } from './clients/hatchet-client';
import WorkflowRunRef from './util/workflow-run-ref';
import { Worker } from './clients/worker';
import { WorkerLabels } from './clients/dispatcher/dispatcher-client';
import { CreateStepRateLimit, RateLimitDuration, WorkerLabelComparator } from './protoc/workflows';
export declare const CreateRateLimitSchema: z.ZodObject<{
    key: z.ZodOptional<z.ZodString>;
    staticKey: z.ZodOptional<z.ZodString>;
    dynamicKey: z.ZodOptional<z.ZodString>;
    units: z.ZodUnion<[z.ZodNumber, z.ZodString]>;
    limit: z.ZodOptional<z.ZodUnion<[z.ZodNumber, z.ZodString]>>;
    duration: z.ZodOptional<z.ZodNativeEnum<typeof RateLimitDuration>>;
}, "strip", z.ZodTypeAny, {
    units: string | number;
    key?: string | undefined;
    duration?: RateLimitDuration | undefined;
    limit?: string | number | undefined;
    staticKey?: string | undefined;
    dynamicKey?: string | undefined;
}, {
    units: string | number;
    key?: string | undefined;
    duration?: RateLimitDuration | undefined;
    limit?: string | number | undefined;
    staticKey?: string | undefined;
    dynamicKey?: string | undefined;
}>;
export declare const DesiredWorkerLabelSchema: z.ZodOptional<z.ZodUnion<[z.ZodString, z.ZodNumber, z.ZodObject<{
    value: z.ZodUnion<[z.ZodString, z.ZodNumber]>;
    required: z.ZodOptional<z.ZodBoolean>;
    weight: z.ZodOptional<z.ZodNumber>;
    comparator: z.ZodOptional<z.ZodNativeEnum<typeof WorkerLabelComparator>>;
}, "strip", z.ZodTypeAny, {
    value: string | number;
    required?: boolean | undefined;
    comparator?: WorkerLabelComparator | undefined;
    weight?: number | undefined;
}, {
    value: string | number;
    required?: boolean | undefined;
    comparator?: WorkerLabelComparator | undefined;
    weight?: number | undefined;
}>]>>;
export declare const CreateStepSchema: z.ZodObject<{
    name: z.ZodString;
    parents: z.ZodOptional<z.ZodArray<z.ZodString, "many">>;
    timeout: z.ZodOptional<z.ZodString>;
    retries: z.ZodOptional<z.ZodNumber>;
    rate_limits: z.ZodOptional<z.ZodArray<z.ZodObject<{
        key: z.ZodOptional<z.ZodString>;
        staticKey: z.ZodOptional<z.ZodString>;
        dynamicKey: z.ZodOptional<z.ZodString>;
        units: z.ZodUnion<[z.ZodNumber, z.ZodString]>;
        limit: z.ZodOptional<z.ZodUnion<[z.ZodNumber, z.ZodString]>>;
        duration: z.ZodOptional<z.ZodNativeEnum<typeof RateLimitDuration>>;
    }, "strip", z.ZodTypeAny, {
        units: string | number;
        key?: string | undefined;
        duration?: RateLimitDuration | undefined;
        limit?: string | number | undefined;
        staticKey?: string | undefined;
        dynamicKey?: string | undefined;
    }, {
        units: string | number;
        key?: string | undefined;
        duration?: RateLimitDuration | undefined;
        limit?: string | number | undefined;
        staticKey?: string | undefined;
        dynamicKey?: string | undefined;
    }>, "many">>;
    worker_labels: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodLazy<z.ZodOptional<z.ZodUnion<[z.ZodString, z.ZodNumber, z.ZodObject<{
        value: z.ZodUnion<[z.ZodString, z.ZodNumber]>;
        required: z.ZodOptional<z.ZodBoolean>;
        weight: z.ZodOptional<z.ZodNumber>;
        comparator: z.ZodOptional<z.ZodNativeEnum<typeof WorkerLabelComparator>>;
    }, "strip", z.ZodTypeAny, {
        value: string | number;
        required?: boolean | undefined;
        comparator?: WorkerLabelComparator | undefined;
        weight?: number | undefined;
    }, {
        value: string | number;
        required?: boolean | undefined;
        comparator?: WorkerLabelComparator | undefined;
        weight?: number | undefined;
    }>]>>>>>;
}, "strip", z.ZodTypeAny, {
    name: string;
    timeout?: string | undefined;
    parents?: string[] | undefined;
    retries?: number | undefined;
    rate_limits?: {
        units: string | number;
        key?: string | undefined;
        duration?: RateLimitDuration | undefined;
        limit?: string | number | undefined;
        staticKey?: string | undefined;
        dynamicKey?: string | undefined;
    }[] | undefined;
    worker_labels?: Record<string, string | number | {
        value: string | number;
        required?: boolean | undefined;
        comparator?: WorkerLabelComparator | undefined;
        weight?: number | undefined;
    } | undefined> | undefined;
}, {
    name: string;
    timeout?: string | undefined;
    parents?: string[] | undefined;
    retries?: number | undefined;
    rate_limits?: {
        units: string | number;
        key?: string | undefined;
        duration?: RateLimitDuration | undefined;
        limit?: string | number | undefined;
        staticKey?: string | undefined;
        dynamicKey?: string | undefined;
    }[] | undefined;
    worker_labels?: Record<string, string | number | {
        value: string | number;
        required?: boolean | undefined;
        comparator?: WorkerLabelComparator | undefined;
        weight?: number | undefined;
    } | undefined> | undefined;
}>;
export type JsonObject = {
    [Key in string]: JsonValue;
} & {
    [Key in string]?: JsonValue | undefined;
};
export type JsonArray = JsonValue[] | readonly JsonValue[];
export type JsonPrimitive = string | number | boolean | null;
export type JsonValue = JsonPrimitive | JsonObject | JsonArray;
export type NextStep = {
    [key: string]: JsonValue;
};
interface ContextData<T, K> {
    input: T;
    parents: Record<string, any>;
    triggered_by: string;
    user_data: K;
}
export declare class ContextWorker {
    private worker;
    constructor(worker: Worker);
    id(): string | undefined;
    hasWorkflow(workflowName: string): boolean;
    labels(): WorkerLabels;
    upsertLabels(labels: WorkerLabels): Promise<WorkerLabels>;
}
export declare class Context<T, K = {}> {
    data: ContextData<T, K>;
    input: T;
    controller: AbortController;
    action: Action;
    client: HatchetClient;
    worker: ContextWorker;
    overridesData: Record<string, any>;
    logger: Logger;
    spawnIndex: number;
    constructor(action: Action, client: HatchetClient, worker: Worker);
    stepOutput(step: string): NextStep;
    triggeredByEvent(): boolean;
    workflowInput(): T;
    workflowName(): string;
    userData(): K;
    stepName(): string;
    workflowRunId(): string;
    retryCount(): number;
    playground(name: string, defaultValue?: string): string;
    log(message: string, level?: LogLevel): void;
    /**
     * Refreshes the timeout for the current step.
     * @param incrementBy - The interval by which to increment the timeout.
     *                     The interval should be specified in the format of '10s' for 10 seconds,
     *                     '1m' for 1 minute, or '1d' for 1 day.
     */
    refreshTimeout(incrementBy: string): Promise<void>;
    releaseSlot(): Promise<void>;
    putStream(data: string | Uint8Array): Promise<void>;
    /**
     * Spawns multiple workflows.
     *
     * @param workflows an array of objects containing the workflow name, input data, and options for each workflow
     * @returns a list of references to the spawned workflow runs
     */
    spawnWorkflows<Q = JsonValue, P = JsonValue>(workflows: Array<{
        workflow: string | Workflow;
        input: Q;
        options?: {
            key?: string;
            sticky?: boolean;
            additionalMetadata?: Record<string, string>;
        };
    }>): Promise<WorkflowRunRef<P>[]>;
    /**
     * Spawns a new workflow.
     *
     * @param workflowName the name of the workflow to spawn
     * @param input the input data for the workflow
     * @param options additional options for spawning the workflow. If a string is provided, it is used as the key.
     *                If an object is provided, it can include:
     *                - key: a unique identifier for the workflow (deprecated, use options.key instead)
     *                - sticky: a boolean indicating whether to use sticky execution
     * @param <Q> the type of the input data
     * @param <P> the type of the output data
     * @return a reference to the spawned workflow run
     */
    spawnWorkflow<Q = JsonValue, P = JsonValue>(workflow: string | Workflow, input: Q, options?: string | {
        key?: string;
        sticky?: boolean;
        additionalMetadata?: Record<string, string>;
    }): WorkflowRunRef<P>;
    additionalMetadata(): Record<string, string>;
    childIndex(): number | undefined;
    childKey(): string | undefined;
    parentWorkflowRunId(): string | undefined;
}
export type StepRunFunction<T, K> = (ctx: Context<T, K>) => Promise<NextStep | void> | NextStep | void;
export interface CreateStep<T, K> extends z.infer<typeof CreateStepSchema> {
    run: StepRunFunction<T, K>;
}
export declare function mapRateLimit(limits: CreateStep<any, any>['rate_limits']): CreateStepRateLimit[];
export {};
